/* -*- c -*- */
/* $Id$ */

/* Copyright (C) 2008 Alexander Chernov <cher@ejudge.ru> */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

typedef long long ej_time64_t;

struct run_header
{
  unsigned char version;
  unsigned char _pad1[19];
  unsigned char byte_order;
  unsigned char _pad2[11];
  ej_time64_t start_time;
  ej_time64_t sched_time;
  ej_time64_t duration;
  ej_time64_t stop_time;
  ej_time64_t finish_time;
  ej_time64_t saved_duration;
  ej_time64_t saved_stop_time;
  unsigned char _pad3[40];
};

static const char *progname = 0;

static void die(const char *format, ...)
  __attribute__((noreturn, format(printf, 1, 2)));
static void die(const char *format, ...)
{
  char buf[1024];
  va_list args;

  va_start(args, format);
  vsnprintf(buf, sizeof(buf), format, args);
  va_end(args);

  fprintf(stderr, "%s: fatal: %s\n", progname, buf);
  exit(1);
}

int main(int argc, char *argv[])
{
  int fd, r, updated = 0;
  struct run_header head;

  progname = argv[0];
  if (argc != 2) die("invalid number of arguments");
  if ((fd = open(argv[1], O_RDWR, 0)) < 0)
    die("cannot open file %s: %s", argv[1], strerror(errno));
  r = read(fd, &head, sizeof(head));
  if (r != sizeof(head))
    die("error reading the log header from %s: %s", argv[1], strerror(errno));
  if (lseek(fd, 0, SEEK_SET) < 0)
    die("lseek on %s failed: %s", argv[1], strerror(errno));

  if (head.duration != 0) {
    printf("duration is %lld, fixing to 0\n", head.duration);
    head.duration = 0;
    updated = 1;
  }
  if (head.stop_time != 0) {
    printf("stop_time is %lld, fixing to 0\n", head.stop_time);
    head.stop_time = 0;
    updated = 1;
  }
  if (head.finish_time != 0) {
    printf("finish_time is %lld, fixing to 0\n", head.finish_time);
    head.finish_time = 0;
    updated = 1;
  }
  if (!updated) {
    printf("no update is needed\n");
    return 0;
  }
  r = write(fd, &head, sizeof(head));
  if (r != sizeof(head))
    die("error writing the log header to %s: %s", argv[1], strerror(errno));
  close(fd);
  printf("fixed\n");

  return 0;
}
